FORMATO_TXT_F931 = {
    'cuil': {'from': 1, 'long': 11, 'type': 'EN'},
    'nombre_completo': {'from': 12, 'long': 30, 'type': 'AL'},
    'conyuge': {'from': 42, 'long': 1, 'type': 'EN'},
    'hijos': {'from': 43, 'long': 2, 'type': 'EN'},
    'situacion': {'from': 45, 'long': 2, 'type': 'EN'},
    'condicion': {'from': 47, 'long': 2, 'type': 'EN'},
    'actividad': {'from': 49, 'long': 3, 'type': 'EN'},
    'zona': {'from': 52, 'long': 2, 'type': 'EN'},
    'porcentaje_aporte_adicional_ss': {'from': 54, 'long': 5, 'type': 'DE'},
    'modalidad_contrato': {'from': 59, 'long': 3, 'type': 'EN'},
    'obra_social': {'from': 62, 'long': 6, 'type': 'EN'},
    'adherentes': {'from': 68, 'long': 2, 'type': 'EN'},
    'remuneracion_total': {'from': 70, 'long': 12, 'type': 'DE'},
    'remuneracion_01': {'from': 82, 'long': 12, 'type': 'DE'},
    'asign_fam': {'from': 94, 'long': 9, 'type': 'DE'},
    'aporte_vol': {'from': 103, 'long': 9, 'type': 'DE'},
    'importe_adicional_os': {'from': 112, 'long': 9, 'type': 'DE'},
    'imp_exc_ap_ss': {'from': 121, 'long': 9, 'type': 'DE'},
    'imp_exc_ap_os': {'from': 130, 'long': 9, 'type': 'DE'},
    'localidad': {'from': 139, 'long': 50, 'type': 'AL'},
    'remuneracion_02': {'from': 189, 'long': 12, 'type': 'DE'},
    'remuneracion_03': {'from': 201, 'long': 12, 'type': 'DE'},
    'remuneracion_04': {'from': 213, 'long': 12, 'type': 'DE'},
    'codigo_siniestrado': {'from': 225, 'long': 2, 'type': 'EN'},
    'corresponde_reduccion': {'from': 227, 'long': 1, 'type': 'EN'},
    'capital_lrt': {'from': 228, 'long': 9, 'type': 'DE'},
    'tipo_empresa': {'from': 237, 'long': 1, 'type': 'EN'},
    'aporte_adicional_os': {'from': 238, 'long': 9, 'type': 'DE'},
    'regimen': {'from': 247, 'long': 1, 'type': 'EN'},
    'situacion_1': {'from': 248, 'long': 2, 'type': 'EN'},
    'dia_sr1': {'from': 250, 'long': 2, 'type': 'EN'},
    'situacion_2': {'from': 252, 'long': 2, 'type': 'EN'},
    'dia_sr2': {'from': 254, 'long': 2, 'type': 'EN'},
    'situacion_3': {'from': 256, 'long': 2, 'type': 'EN'},
    'dia_sr3': {'from': 258, 'long': 2, 'type': 'EN'},
    'sueldo': {'from': 260, 'long': 12, 'type': 'DE'},
    'sac': {'from': 272, 'long': 12, 'type': 'DE'},
    'hs_extras': {'from': 284, 'long': 12, 'type': 'DE'},
    'zona_desfavorable': {'from': 296, 'long': 12, 'type': 'DE'},
    'vacaciones': {'from': 308, 'long': 12, 'type': 'DE'},
    'k_dias': {'from': 320, 'long': 9, 'type': 'DE'},
    'remuneracion_05': {'from': 329, 'long': 12, 'type': 'DE'},
    'convencionado': {'from': 341, 'long': 1, 'type': 'EN'},
    'remuneracion_06': {'from': 342, 'long': 12, 'type': 'DE'},
    'tipo_operacion': {'from': 354, 'long': 1, 'type': 'EN'},
    'adicionales': {'from': 355, 'long': 12, 'type': 'DE'},
    'premios': {'from': 367, 'long': 12, 'type': 'DE'},
    'remuneracion_08': {'from': 379, 'long': 12, 'type': 'DE'},
    'remuneracion_07': {'from': 391, 'long': 12, 'type': 'DE'},
    'k_hs_extras': {'from': 403, 'long': 3, 'type': 'EN'},
    'no_remunerativo': {'from': 406, 'long': 12, 'type': 'DE'},
    'maternidad': {'from': 418, 'long': 12, 'type': 'DE'},
    'rectificacion': {'from': 430, 'long': 9, 'type': 'DE'},
    'remuneracion_09': {'from': 439, 'long': 12, 'type': 'DE'},
    'porc_contr_dif_ss': {'from': 451, 'long': 9, 'type': 'DE'},
    'k_horas': {'from': 460, 'long': 3, 'type': 'EN'},
    'seguro_vida_obligatorio': {'from': 463, 'long': 1, 'type': 'BO'},
    'detraccion': {'from': 464, 'long': 12, 'type': 'DE'},
    'incremento': {'from': 476, 'long': 12, 'type': 'DE'},
    'remuneracion_11': {'from': 488, 'long': 12, 'type': 'DE'},
}


def get_value_from_txt(txt_line: str, field_name: str) -> str:
    """ Retorna el valor de un campo en un txt de F931 de acuerdo a las
        posiciones del campo en el txt detalladas en FORMATO_TXT_F931
    """
    resp = ''

    if field_name in FORMATO_TXT_F931:
        resp = txt_line[FORMATO_TXT_F931[field_name]['from'] - 1:FORMATO_TXT_F931[field_name]['from'] - 1 +
                        FORMATO_TXT_F931[field_name]['long']].strip()

    return resp



from py_arg_reports.reporters.f931.tools import FORMATO_TXT_F931
from py_arg_reports.tools.tools import sync_format


def genera_txt_f931(
    json_data: dict,
    output_path: str,
    filename: str = None,
) -> tuple:
    """ Genera el archivo de texto para la F931,
        Args:
            json_data: dict, datos de F931 Fijos
            output_path: str, ruta donde se guardará el archivo
            filename: str, nombre del archivo

        Retorna una tupla:
         - False, error: si falla
         - True, None: si todo salió bien
    """
    resp = None

    if not json_data.get('txt_empleados'):
        return False, 'No se puede generar el txt para el F931, no hay datos'

    # Configurar filename si usuario no lo especifica
    if not filename:
        filename = f'txt_f931_{json_data["cuit"]}_{json_data["periodo"]}'

    # Crear archivo vacio
    full_path = f'{output_path}/{filename}.txt'
    with open(full_path, 'w') as f:
        f.write('')

    # Comienzo dato por dato de json_data a generar en el txt linea por linea
    for empleado in json_data['txt_empleados']:
        # Generar linea por empleado
        line = ''
        for field_name in FORMATO_TXT_F931:
            if field_name not in empleado.keys():
                return False, f'El campo {field_name} no fue encontrado en los datos'
            tipo_dato = FORMATO_TXT_F931[field_name]['type']
            if tipo_dato == 'DE':
                multiplicador = FORMATO_TXT_F931[field_name].get('multiplicador', 100)
            else:
                multiplicador = FORMATO_TXT_F931[field_name].get('multiplicador', 1)
            largo = FORMATO_TXT_F931[field_name]['long']
            info = empleado[field_name]
            info_formatted = sync_format(str(info), largo, tipo_dato, multiplicador)

            # Situaciones de Revista 2 y 3 no pueden ser vacías o "00", cambiar a "01" si es así
            if field_name in ['situacion_2', 'situacion_3'] and (info_formatted == '00' or info_formatted == '  '):
                info_formatted = '01'

            line += info_formatted
        # Agregar salto de linea
        line += '\n'
        # Escribir linea en el archivo
        with open(full_path, 'a', encoding='latin-1') as f:
            f.write(line)

    return True, resp



import json
import os
import tempfile
import unittest

from py_arg_reports.reporters.libro_sueldo_digital.reporter import genera_txt_lsd


class TestGeneracionLSD(unittest.TestCase):
    """ Testing para Generación Libro Sueldos Digital
    """
    @classmethod
    def setUpClass(cls):
        # Specify the path where the temporary folder should be created
        temp_folder_path = './tests/temp/'
        # Create a temporary folder for testing in the specified path
        cls.temp_folder = tempfile.mkdtemp(dir=temp_folder_path)
        if cls.temp_folder[-1] != '/':
            cls.temp_folder += '/'

        with open('./py_arg_reports/test_cases/lsd-info-1l.json', 'r') as f:
            cls.json_data_1l = json.load(f)
        with open('./py_arg_reports/test_cases/lsd-info-2l.json', 'r') as f:
            cls.json_data_2l = json.load(f)
        with open('./py_arg_reports/test_cases/lsd-info-missing_conceptos.json', 'r') as f:
            cls.missing_conceptos = json.load(f)
        with open('./py_arg_reports/test_cases/lsd-info-missing_empleados.json', 'r') as f:
            cls.missing_empleados = json.load(f)
        with open('./py_arg_reports/test_cases/lsd-info-missing_info931.json', 'r') as f:
            cls.missing_info931 = json.load(f)
        with open('./py_arg_reports/test_cases/lsd-info-missing_info931_campos.json', 'r') as f:
            cls.missing_info931_campos = json.load(f)
        cls.empty_json = {}
        cls.keys_missing_json = {
            "periodo": "2024-03-01",
            "empresa": "GLADYS LUCAS HERNAN",
            "cuit": "20123456780",
            "cantidad_liquidaciones": 1
        }

    @classmethod
    def tearDownClass(cls):
        # Clean up: Delete the temporary folder and its contents
        if os.path.exists(cls.temp_folder):
            for root, dirs, files in os.walk(cls.temp_folder, topdown=False):
                for file in files:
                    os.remove(os.path.join(root, file))
                for dir in dirs:
                    os.rmdir(os.path.join(root, dir))
            os.rmdir(cls.temp_folder)

    def test_descarga_lsd_1(self):
        """ Prueba de la generacion de txt de Libro Sueldos Digital con 1 liquidacion
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.json_data_1l,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (True, None))
        # Le agrega el "_1" por ser liquidacion 1
        full_path = self.temp_folder + 'txt_lsd_prueba_1.txt'
        print("full_path: ", full_path)

        # Check if the file exists
        self.assertTrue(os.path.exists(full_path))

        # Check the number of lines in the txt file
        with open(self.temp_folder + 'txt_lsd_prueba_1.txt', 'r') as file:
            # Recorro las líneas del archivo, los 2 primeros caracteres de cada línea son el tipo de registro
            # Deben tener este largo:
            # '01': Datos referenciales del envío -> Largo 35
            # '02': Datos referenciales de la Liquidación de SyJ del trabajador -> Largo 115
            # '03': Detalle de los conceptos de sueldo liquidados al trabajador -> Largo 51
            # '04': Datos del trabajador para el calculo de la DJ F931 -> Largo 370
            # '05': Datos de Eventuales -> Largo 65
            lines = file.readlines()
            # Tienen que ser 5 líneas porque tiene:
            # 1 línea de tipo 01
            # 1 línea de tipo 02
            # 13 línea de tipo 03
            # 1 línea de tipo 04
            self.assertEqual(len(lines), 16)

        for line in lines:
            # le remuevo el salto de carro si tiene
            line = line.replace('\n', '')

            if line[:2] == '01':
                self.assertEqual(len(line), 35)
            elif line[:2] == '02':
                self.assertEqual(len(line), 115)
            elif line[:2] == '03':
                self.assertEqual(len(line), 51)
            elif line[:2] == '04':
                self.assertEqual(len(line), 370)
            elif line[:2] == '05':
                self.assertEqual(len(line), 65)

    def test_descarga_empty_json(self):
        """ Prueba la descarga del archivo para un json vacío
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.empty_json,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (False, 'No se puede generar el txt para Libro Sueldo Digital, no hay datos'))
        full_path = self.temp_folder + 'txt_lsd_prueba_1.txt'

        # Check if the file not exists
        self.assertFalse(os.path.exists(full_path))

    def test_key_missing_conceptos(self):
        """ Prueba la descarga del archivo para un json con conceptos faltantes
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.missing_conceptos,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (False, 'Falta la clave conceptos_liquidados en el diccionario de empleados'))

    def test_key_missing_infof931(self):
        """ Prueba la descarga del archivo para un json con info_f931 faltante
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.missing_info931,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (False, 'Falta la clave info_f931 en el diccionario de empleados'))

    def test_key_missing_infof931_campos(self):
        """ Prueba la descarga del archivo para un json con algunos campos de info_f931 faltantes
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.missing_info931_campos,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (False, 'Falta el campo convencionado en el diccionario de info_931'))

    def test_broken_json(self):
        """ Prueba la descarga del archivo para un json roto
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.keys_missing_json,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        self.assertEqual(resp_descarga, (False, 'No se puede generar el txt, dato esencial liquidaciones no encontrado'))

    def test_key_missing_empleados(self):
        """ Prueba la descarga del archivo para un json con empleados faltantes
        """
        resp_descarga = genera_txt_lsd(
            json_data=self.missing_empleados,
            output_path=self.temp_folder,
            filename='txt_lsd_prueba',
        )

        expected_resp = 'No se puede generar el txt, dato esencial empleados_liquidados no encontrado en liquidaciones'
        self.assertEqual(resp_descarga, (False, expected_resp))


if __name__ == '__main__':
    unittest.main()



import os
import zipfile


def integer_to_amount_txt(amount: int, long: int, multiplicador: int = 1) -> str:
    """ Convierte un monto entero a formato de texto
    """
    resp = amount * multiplicador
    resp = str(resp).zfill(long)
    return resp


def float_to_amount_txt(amount: float, long: int, multiplicador: int = 100, no_coma: bool = False) -> str:
    """ Convierte un monto float a formato de texto
    """
    resp = amount * multiplicador
    if not no_coma:
        resp = "{:.2f}".format(amount)
        resp = resp.zfill(long).replace('.', ',')
    else:
        resp = int(resp)
        resp = str(resp)
        resp = resp.zfill(long)
    return resp


def amount_txt_to_integer(amount_txt: str, mulitp=100) -> int:
    """ Convierte un monto en formato de texto a entero
    """
    resp = float(amount_txt.replace(',', '.')) * mulitp
    resp = int(resp)

    return resp


def amount_txt_to_float(amount_txt: str, multip: int = 100, rount_to: int = 2) -> float:
    """ Convierte un monto en formato de texto a float
    """
    resp = float(amount_txt.replace(',', '.')) * multip
    resp = float(resp)
    resp = round(resp, rount_to)

    return resp


def sync_format(info: str, expected_len: int, type_info: str, multiplicador: int = 1, no_coma: bool = False) -> str:
    """ Sincroniza el formato de un campo de un txt de F931
        Args:
            info: str, valor del campo
            expected_len: int, longitud esperada del campo
            type_info: str, tipo de campo, estos pueden ser:
                EN - Entero
                DE - Decimal
                AL - Alfabético
                AN - Alfanumérico
                BO - Booleano
            multiplicador: int, multiplicador del campo
    """
    resp = info
    if type_info == 'BO':
        resp = '1' if info else '0'
        return resp

    if len(info) != expected_len or ',' in info:
        if len(info) > expected_len and not type_info == 'BO':
            resp = round(float(info.replace(',', '.').strip()))
            # Ver si está ok multiplicar por 100
            resp = str(resp * multiplicador).zfill(expected_len)
        else:
            if type_info == 'DE':
                resp = float_to_amount_txt(float(info), expected_len, multiplicador, no_coma)
            elif type_info == 'EN':
                resp = integer_to_amount_txt(int(info), expected_len, multiplicador)

    # En los otros casos (AL o AN) se completa con espacios a la derecha
    if type_info in ['AL', 'AN']:
        resp = str(resp).ljust(expected_len)

    return resp


def convert_to_float_if_possible(value_str: str):
    """Convert the value_str to float if possible, else return the value_str
    """
    try:
        return float(value_str)
    except ValueError:
        return value_str


def convert_to_int_if_possible(value_str: str):
    """Convert the value_str to int if possible, else return the value_str
    """
    try:
        return int(value_str)
    except ValueError:
        return value_str


def file_compress(inp_file_names, out_zip_file):
    """
    function : file_compress
    args : inp_file_names : list of filenames to be zipped
    out_zip_file : output zip file
    return : none
    assumption : Input file paths and this code is in same directory.
    """
    # Select the compression mode ZIP_DEFLATED for compression
    # or zipfile.ZIP_STORED to just store the file
    compression = zipfile.ZIP_DEFLATED
    print(f" *** Input File name passed for zipping - {inp_file_names}")

    # create the zip file first parameter path/name, second mode
    print(f' *** out_zip_file is - {out_zip_file}')
    zf = zipfile.ZipFile(out_zip_file, mode="w")

    try:
        for file_to_write in inp_file_names:
            # Add file to the zip file
            # first parameter file to zip, second filename in zip
            file_to_write_name = file_to_write.split('/')[-1]
            print(f' *** Processing file {file_to_write_name}')
            zf.write(file_to_write, file_to_write_name, compress_type=compression)

    except FileNotFoundError as e:
        print(f' *** Exception occurred during zip process - {e}')
    finally:
        # Don't forget to close the file!
        zf.close()


def delete_list_of_liles(list_to_delete: list):
    for f in list_to_delete:
        fname = f.rstrip()
        if os.path.isfile(fname):
            os.remove(fname)



from pathlib import Path
from py_arg_reports.logs import get_logger
from py_arg_reports.reporters.libro_sueldo.data import translate_data
from py_arg_reports.tools.pdf import CanvasPDF, CanvaPDFBlock, Format, Rect
from py_arg_reports.tools.recibos_utils import float_to_format_currency
from reportlab.lib.units import cm


log = get_logger(__name__)
F10 = Format(font_size=10)
F7 = Format(font_size=7)

t10_line_sep = 0.5
t9_line_sep = 0.3


def descargar_libro(json_data: dict, output_path: str, filename: str) -> tuple:
    """ Descarga el libro sueldo en formato PDF,
        Retorna una tupla:
         - False, error: si falla
         - True, None: si todo salió bien
    """

    try:
        info_recibo = translate_data(json_data)
    except Exception as e:
        error_detail = f'Error al traducir los datos para libro sueldo: {str(e)}'
        log.error(error_detail)
        return False, error_detail

    # Cada liquidación va a tener su propia carpeta en download
    if not Path(output_path).exists():
        # create the folder
        Path(output_path).mkdir(parents=True, exist_ok=True)
    my_file_path = Path(output_path) / f'{filename}.pdf'
    log.info(f'Generando libro sueldo en {my_file_path}')

    # Mi PDF general donde voy a agregar los bloques
    PDF = CanvasPDF(
        file_path=str(my_file_path),
        title='Libro Sueldo', units=cm,
        data=info_recibo,
    )
    log.info(f'Creando PDF en {my_file_path}')

    draw_header(PDF)
    draw_footer(PDF)

    pos_y = PDF.last_y + 0.1
    empleado_h = 5
    total_rem = 0
    total_no_rem = 0
    total_desc = 0
    for empleado in info_recibo['empleados']:
        log.info(f'Generando empleado {empleado["legajo"]}')
        draw_empleado(PDF, empleado, start_y=pos_y, height=empleado_h)
        pos_y = PDF.last_y + 0.3
        total_rem += empleado['totales_liquidacion']['total_remunerativo']
        total_no_rem += empleado['totales_liquidacion']['total_no_remunerativo']
        total_desc += empleado['totales_liquidacion']['total_retenciones']

    # Agregar bloque de totales
    totales = CanvaPDFBlock(PDF, Rect(0, pos_y, 0, 3), Format(font_size=10, fill_color='#F0F0F099'))
    totales.text('Totales', bold=True, x=1, y=0.5, format_=F7)
    total_empleados = len(info_recibo['legajos'])
    totales.text(f'Cantidad de empleados: {total_empleados}', bold=True, x=12, y=0.5, format_=F7)

    total_rem2 = float_to_format_currency(total_rem)
    totales.text(f'Remunerativos {total_rem2}', bold=True, x=1, y=1.5)
    total_no_rem2 = float_to_format_currency(total_no_rem, 2)
    totales.text(f'No Remunerativos {total_no_rem2}', bold=True, x=7.3, y=1.5)
    total_desc2 = float_to_format_currency(total_desc)
    totales.text(f'Descuentos {total_desc2}', bold=True, x=14, y=1.5)

    neto = total_rem + total_no_rem - total_desc
    neto = float_to_format_currency(neto)
    totales.text(f'Neto a cobrar {neto}', bold=True, x=1, y=2.5, format_=F7)

    PDF.finish_page()
    PDF.save()
    return True, None


def draw_header(PDF: CanvasPDF):
    info_recibo = PDF.data
    log.info(f'Creando header en pagina {PDF.page}')
    # Estimar el alto que va a tener esto, necesito las actividades que son las que me pueden hacer variar esto
    # actividades
    actividad_principal = info_recibo.get('actividad_principal', {})
    actividades_secundarias = info_recibo.get('actividades_secundarias', [])
    total_act_secundarias = len(actividades_secundarias)
    initial_h = (
        1 +  # el header principal con el titulo
        0.4 +  # el nombre de la empresa
        0.6 +  # el domicilio de la empresa
        0.6 +  # la actividad principal
        (0.3 * total_act_secundarias)  # por cada actividad secundaria
    )
    # Agregar el bloque de headers izquiero
    header = CanvaPDFBlock(
        PDF,
        is_header=True,
        rect=Rect(0, 0, 0, initial_h),
        format_=Format(font_size=10, fill_color='#D0D0D0CC')
    )

    # Agregar contenido al header
    header.text('Hojas Móviles Libro Art. 52 Ley 20744', align='center', y=0.7, format_=F10, bold=True)
    col = [info_recibo['company_name'], info_recibo['domicilio']]
    header.text_column(col, start_x=0.4, start_y=1.3, format_=F7, bold=True)

    # Verificar que haya al menos una actividad principal
    if actividad_principal:
        apri_label = 'Actividad principal: '
        apri_name = actividad_principal.get('name', 'No especificada')

        # Mostrar la actividad principal
        header.text(apri_label, x=0.4, y=2.2, format_=F7, bold=True)
        header.text(apri_name, x=0.4 + 2.5, y=2.2, format_=F7)

        # Mostrar actividades secundarias si existen
        if actividades_secundarias:
            asec_label = 'Actividades secundarias:'
            header.text(asec_label, x=0.4, y=2.7, format_=F7, bold=True)

            for i, actividad in enumerate(actividades_secundarias):
                asec_name = actividad.get('name', 'No especificada')
                y_pos = 2.55 + (i + 0.5) * 0.3
                header.text(asec_name, x=0.4 + 3.19, y=y_pos, format_=F7)
    else:
        header.text(' Actividades no especificadas', x=0.4, y=2.2, format_=F7)

    # Agregar el bloque de headers derecho
    per_label = 'Periodo: '
    per_content = f'{info_recibo["tipo_liquidacion"]} {info_recibo["periodo"]}'
    cuit_label = 'CUIT: '
    cuit_content = info_recibo['cuit']
    col = [cuit_label, per_label]
    header.text_column(col, start_x=13, start_y=1.3, format_=F7, bold=True)
    header.text_column([cuit_content, per_content], start_x=14.4, start_y=1.3, format_=F7)

    return header


def draw_footer(PDF: CanvasPDF):
    log.info(f'Creando footer en pagina {PDF.page}')
    footer = CanvaPDFBlock(
        PDF, is_footer=True,
        rect=Rect(0, PDF.height-1, 0, 0.5), format_=Format(font_size=7, fill_color='#D0D0D0')
    )
    footer.text('Hojas Móviles Libro Art. 52 Ley 20744 -pagina {page}', align='center', y=0.3)
    return footer


def draw_empleado(PDF: CanvasPDF, empleado: dict, start_y, height):

    conceptos = empleado["conceptos_liquidados"]
    # Los conceptos se dividen por tipo
    remunerativos = [c for c in conceptos if c['tipo_concepto'] == 1]
    total_remu = len(remunerativos)
    no_remunerativos = [c for c in conceptos if c['tipo_concepto'] == 2]
    total_no_remu = len(no_remunerativos)
    descuentos = [c for c in conceptos if c['tipo_concepto'] == 3]
    total_desc = len(descuentos)
    # el total es igual al mayor de los tres
    total_c = max(total_remu, total_no_remu, total_desc)

    e_initial_h = (
        1 +  # el nombre del empleado
        0.5 +  # la información básica del empleado
        0.5 +  # la división automática del contrato
        0.5 +  # la información adicional del empleado
        (0.3 * total_c) +  # los conceptos
        1.2  # el neto a cobrar
    )

    empleado_block = CanvaPDFBlock(PDF, Rect(0, start_y, 0, e_initial_h), Format(font_size=7, fill_color='#F0F0F033'))
    name = f'{empleado["legajo"]} - {empleado["nombre"]}'
    y = 0.5
    empleado_block.text(name, bold=True, x=1, y=y)
    # Información básica del empleado
    lista = ['F. Ingreso', '', 'Remun. asignada']
    y = y + t9_line_sep + 0.1
    empleado_block.text_column(lista, start_x=1, start_y=y, line_sep=t9_line_sep, format_=F7, bold=True)
    lista = [empleado["fecha_ingreso"], empleado["fecha_ingreso_2"], empleado["basico"]]
    empleado_block.text_column(lista, start_x=3.5, start_y=y, line_sep=t9_line_sep, format_=F7)
    # División automática del contrato
    contrato_text = empleado["contrato"]

    lista = ['Estado civil', 'Puesto', 'Mod.Contrato']
    empleado_block.text_column(lista, start_x=6, start_y=y, line_sep=t9_line_sep, format_=F7, bold=True)
    lista = [empleado['estado_civil'], empleado["posicion"], contrato_text]
    empleado_block.text_column(lista, start_x=8, start_y=y, line_sep=t9_line_sep, format_=F7)
    # Información adicional del empleado
    lista = ['CUIL', 'Categoría', 'Seccion']
    y = 0.5 + t9_line_sep + 0.1
    empleado_block.text_column(lista, start_x=13.5, start_y=y, line_sep=t9_line_sep, format_=F7, bold=True)
    lista = [empleado["cuil"], empleado["categoria"], empleado["area"]]
    empleado_block.text_column(lista, start_x=15, start_y=y, line_sep=t9_line_sep, format_=F7)

    empleado_block.line(Rect(2, 2, 16, 1.9), line_with=2)

    y_titles = 2.3
    # Los tipos de conceptos son:
    # 1 Remunerativos
    # 2 No Remunerativos
    # 3 Descuentos

    def name_cant(cpt, max_length=30):
        """ Devuelve el nombre del concepto con la cantidad si la tiene """
        cantidad = cpt.get('cantidad')
        final = cpt['name']
        # sie l nombre es muy largo, cortarlo
        if len(final) > max_length:
            half_length = (max_length - 4) // 2
            final = f'{final[:half_length]}....{final[-half_length:]}'

        if cantidad:
            final = f'{final} ({cantidad})'
        return final

    # REMUNERATIVOS

    # Buscar todos los conceptos remunerativos (tipo_concepto=1)
    empleado_block.text('Remunerativos', bold=True, x=0.3, y=y_titles, format_=F7)
    total_remu = sum([cpt['importe'] for cpt in remunerativos])
    empleado_block.text(float_to_format_currency(total_remu), x=5.5, y=y_titles, align='right', bold=True,  format_=F7)
    lista = [name_cant(cpt) for cpt in remunerativos]
    empleado_block.text_column(lista, start_x=0.3, start_y=y_titles + 0.5, line_sep=t9_line_sep, format_=F7)
    lista = [float_to_format_currency(cpt['importe'], include_currency=False) for cpt in remunerativos]
    empleado_block.text_column(lista, start_x=5.5, start_y=y_titles + 0.5, align='right', line_sep=t9_line_sep, format_=F7)

    # NO REMUNERATIVOS

    # Buscar todos los conceptos no remunerativos (tipo_concepto=2)
    empleado_block.text('No Remunerativos', bold=True, x=6, y=y_titles, format_=F7)
    total_no_remu = sum([cpt['importe'] for cpt in no_remunerativos])
    empleado_block.text(float_to_format_currency(total_no_remu), x=12, y=y_titles, align='right', bold=True, format_=F7)

    lista = [name_cant(cpt) for cpt in no_remunerativos]
    empleado_block.text_column(lista, start_x=6, start_y=y_titles + 0.5, line_sep=t9_line_sep, format_=F7)
    lista = [float_to_format_currency(cpt['importe'], include_currency=False) for cpt in no_remunerativos]
    empleado_block.text_column(lista, start_x=12, start_y=y_titles + 0.5, align='right', line_sep=t9_line_sep, format_=F7)

    # DESCUENTOS

    # Buscar todos los conceptos descuentos (tipo_concepto=3)
    empleado_block.text('Descuentos', bold=True, x=12.4, y=y_titles, format_=F7)
    total_desc = sum([cpt['importe'] for cpt in descuentos])
    empleado_block.text(float_to_format_currency(total_desc), x=18.7, y=y_titles, align='right', bold=True, format_=F7)

    lista = [name_cant(cpt) for cpt in descuentos]
    empleado_block.text_column(lista, start_x=12.4, start_y=y_titles + 0.5, line_sep=t9_line_sep, format_=F7)
    lista = [float_to_format_currency(cpt['importe'], include_currency=False) for cpt in descuentos]
    empleado_block.text_column(lista, start_x=18.7, start_y=y_titles + 0.5, align='right', line_sep=t9_line_sep, format_=F7)

    # NETO

    y_titles = e_initial_h - 2.4

    empleado_block.rectangle(Rect(1, y_titles+1.6, 5, 0.6), fill_color='#D0D0D0AA')
    neto_a_cobrar = float_to_format_currency(empleado["totales_liquidacion"]["neto_liquidacion"])
    empleado_block.text(f'Neto a cobrar   {neto_a_cobrar}', bold=True, x=1.2, y=y_titles+2, format_=F7)


if __name__ == '__main__':
    """
    Esto es para hacer pruebas rapido con los archivos de ejemplo.
    Dejar para hacer correcciones rápidas.
    En estas pruebas escupir los logs en la consola
    """
    import json
    # Descargar el libro
    json_data_test_file = 'py_arg_reports/reporters/libro_sueldo/samples/samples-recibo-info.json'
    json_data = json.load(open(json_data_test_file))
    output_path = 'download/'
    filename = 'libro-sueldo'
    descargar_libro(json_data, output_path, filename)



FORMATO_TXT_LSD = {
    'registro1': {
        'tipo_registro': {'from': 1, 'long': 2, 'type': 'AL'},
        'cuit': {'from': 3, 'long': 11, 'type': 'EN'},
        'identificacion_envio': {'from': 14, 'long': 2, 'type': 'AL'},
        'periodo': {'from': 16, 'long': 6, 'type': 'EN'},
        'tipo_liquidacion': {'from': 22, 'long': 1, 'type': 'AL'},
        'numero_liquidacion': {'from': 23, 'long': 5, 'type': 'EN'},
        'dias_base': {'from': 28, 'long': 2, 'type': 'AL'},
        'cantidad_trabajadores': {'from': 30, 'long': 6, 'type': 'EN'},
    },
    'registro2': {
        'tipo_registro': {'from': 1, 'long': 2, 'type': 'AL'},
        'cuil': {'from': 3, 'long': 11, 'type': 'EN'},
        'legajo': {'from': 14, 'long': 10, 'type': 'EN'},
        'dependencia': {'from': 24, 'long': 50, 'type': 'AL'},
        'cbu': {'from': 74, 'long': 22, 'type': 'EN'},
        'dias_tope': {'from': 96, 'long': 3, 'type': 'EN'},
        'fecha_pago': {'from': 99, 'long': 8, 'type': 'FE'},
        'fecha_rubrica': {'from': 107, 'long': 8, 'type': 'FE'},
        'forma_pago': {'from': 115, 'long': 1, 'type': 'AL'},
    },
    'registro3': {
        'tipo_registro': {'from': 1, 'long': 2, 'type': 'AL'},
        'cuil': {'from': 3, 'long': 13, 'type': 'EN'},
        'codigo_concepto': {'from': 16, 'long': 10, 'type': 'EN'},
        'cantidad': {'from': 26, 'long': 5, 'type': 'EN'},
        'unidades': {'from': 31, 'long': 1, 'type': 'EN'},
        'importe': {'from': 32, 'long': 15, 'type': 'DE'},
        'indicador': {'from': 47, 'long': 1, 'type': 'AL'},
        'periodo_ajuste': {'from': 48, 'long': 6, 'type': 'EN'},
    },
    'registro4': {
        'tipo_registro': {'from': 1, 'long': 2, 'type': 'AL'},
        'cuil': {'from': 3, 'long': 11, 'type': 'EN'},
        'conyuge': {'from': 14, 'long': 1, 'type': 'EN'},
        'hijos': {'from': 15, 'long': 2, 'type': 'EN'},
        'convencionado': {'from': 17, 'long': 1, 'type': 'AL'},
        'seguro_vida_obligatorio': {'from': 18, 'long': 1, 'type': 'AL'},
        'corresponde_reduccion': {'from': 19, 'long': 1, 'type': 'AL'},
        'tipo_empresa': {'from': 20, 'long': 1, 'type': 'AL'},
        'tipo_operacion': {'from': 21, 'long': 1, 'type': 'AL'},
        'situacion': {'from': 22, 'long': 2, 'type': 'AL'},
        'condicion': {'from': 24, 'long': 2, 'type': 'AL'},
        'actividad': {'from': 26, 'long': 3, 'type': 'AL'},
        'modalidad_contrato': {'from': 29, 'long': 3, 'type': 'AL'},
        'codigo_siniestrado': {'from': 32, 'long': 2, 'type': 'AL'},
        'zona': {'from': 34, 'long': 2, 'type': 'AL'},
        'situacion_1': {'from': 36, 'long': 2, 'type': 'AL'},
        'dia_sr1': {'from': 38, 'long': 2, 'type': 'EN'},
        'situacion_2': {'from': 40, 'long': 2, 'type': 'AL'},
        'dia_sr2': {'from': 42, 'long': 2, 'type': 'EN'},
        'situacion_3': {'from': 44, 'long': 2, 'type': 'AL'},
        'dia_sr3': {'from': 46, 'long': 2, 'type': 'EN'},
        'k_dias': {'from': 48, 'long': 2, 'type': 'EN'},
        'k_horas': {'from': 50, 'long': 3, 'type': 'EN'},
        'porcentaje_aporte_adicional_ss': {'from': 53, 'long': 5, 'type': 'DE'},
        'porc_contr_dif_ss': {'from': 58, 'long': 5, 'type': 'DE'},
        'obra_social': {'from': 63, 'long': 6, 'type': 'AL'},
        'adherentes': {'from': 69, 'long': 2, 'type': 'EN'},
        'aporte_adicional_os': {'from': 71, 'long': 15, 'type': 'DE'},
        'importe_adicional_os': {'from': 86, 'long': 15, 'type': 'DE'},
        'base_diferencial_aos': {'from': 101, 'long': 15, 'type': 'DE'},
        'base_diferencial_cos': {'from': 116, 'long': 15, 'type': 'DE'},
        'base_calculo_diferencial_lrt': {'from': 131, 'long': 15, 'type': 'DE'},
        'maternidad': {'from': 146, 'long': 15, 'type': 'DE'},
        'remuneracion_total': {'from': 161, 'long': 15, 'type': 'DE'},
        'remuneracion_01': {'from': 176, 'long': 15, 'type': 'DE'},
        'remuneracion_02': {'from': 191, 'long': 15, 'type': 'DE'},
        'remuneracion_03': {'from': 206, 'long': 15, 'type': 'DE'},
        'remuneracion_04': {'from': 221, 'long': 15, 'type': 'DE'},
        'remuneracion_05': {'from': 236, 'long': 15, 'type': 'DE'},
        'remuneracion_06': {'from': 251, 'long': 15, 'type': 'DE'},
        'remuneracion_07': {'from': 266, 'long': 15, 'type': 'DE'},
        'remuneracion_08': {'from': 281, 'long': 15, 'type': 'DE'},
        'remuneracion_09': {'from': 296, 'long': 15, 'type': 'DE'},
        'base_calculo_diferencial_aporte_ss': {'from': 311, 'long': 15, 'type': 'DE'},
        'base_calculo_diferencial_contribucion_ss': {'from': 326, 'long': 15, 'type': 'DE'},
        'remuneracion_10': {'from': 341, 'long': 15, 'type': 'DE'},
        'detraccion': {'from': 356, 'long': 15, 'type': 'DE'},
    },
    'registro5': {
        'tipo_registro': {'from': 1, 'long': 2, 'type': 'AL'},
        'cuil': {'from': 3, 'long': 11, 'type': 'EN'},
        'categoria_profesional': {'from': 14, 'long': 6, 'type': 'EN'},
        'puesto_desempenado': {'from': 20, 'long': 4, 'type': 'AL'},
        'fecha_ingreso': {'from': 24, 'long': 8, 'type': 'FE'},
        'fecha_egreso': {'from': 32, 'long': 8, 'type': 'FE'},
        'remuneracion': {'from': 40, 'long': 15, 'type': 'DE'},
        'cuit_empleador': {'from': 55, 'long': 11, 'type': 'EN'},
    },
}


INDICE_TIPO_LIQUIDACIONES = {
    'Mensual': 'M',
    'Vacaciones': 'M',
    'Quincenal': 'Q',
    '1° Quincena': 'Q',
    '2° Quincena': 'Q',
    'Semanal': 'S',
    'SAC': 'M',
    'Liquidación Final': 'M',
    'Gratificaciones': 'M',
    'Adelantos': 'M',
    'Otros': 'M',
}


def get_value_from_txt(txt_line: str, nro_reg: int, field_name: str) -> str:
    """ Retorna el valor de un campo en un txt de Libro Sueldo Digital de acuerdo a las
        especificaciones de AFIP.
    """
    resp = ''
    where_to_look = FORMATO_TXT_LSD['registro' + str(nro_reg)]

    if field_name in where_to_look:
        resp = txt_line[where_to_look[field_name]['from'] - 1:where_to_look[field_name]['from'] - 1 +
                        where_to_look[field_name]['long']].strip()

    return resp



from py_arg_reports.reporters.acreditaciones import AcreditacionesHeadDetailTrailerFile
from py_arg_reports.tools.txt import fixed_width_str, str_dec_num_to_no_dec_sep


class AcreditacionSantander(AcreditacionesHeadDetailTrailerFile):
    """ Un archivo para el banco Santander """

    def __init__(self, data):
        super().__init__(
            data,
            empresa_required=True,
            liquidacion_required=True,
            empleados_require=['nombre', 'apellido', 'cuil', 'legajo', 'direccion', 'importe_pago', 'cbu'],
            empresa_require=['razon_social', 'cuit'],
            liquidacion_require=['periodo_anio', 'periodo_mes', 'fecha_pago_anio', 'fecha_pago_mes', 'fecha_pago_dia'],
            extras_require=['nro_de_acuerdo', 'codigo_forma_pago'],
        )

    def generate_header(self):
        """ Generar el header """
        ret = "H"
        ret += self.empresa.get('cuit')
        ret += "0"
        ret += "011"  # Codigo de producto 011 es haberes y 013 es honorarios
        ret += self.get_nro_de_acuerdo()  # Nro de acuerdo (asignado por el banco)
        ret += "007"  # Codigo de canal 007 Online Banking Cash Management 005 Diskette
        ret += "00001"  # Numero de envio: suponemos que se manda un solo envio por dia
        ret += "00000"  # Reservado para usos futuros
        ret += " " * 7  # Reservado para usos futuros
        ret += "S"  # Validacion CUIL
        ret += " " * 611  # Reservado para usos futuros
        return ret

    def generate_detalle_empleado(self, empleado):
        """ Generar el empleado """
        # 1 Tipo de registro = D detalle
        ret = 'D'
        # 2 Reservado para usos futuros, Alf, 1 = espacio
        ret += ' '
        # 3 Moneda 0=pesos, 2=dolares, 8=bimonetaira
        ret += '0'
        # 4 Numero del beneficiario (interno empresa) Alf
        # 15 caracters fijos
        legajo_str = str(empleado.get('legajo'))
        ret += fixed_width_str(legajo_str, 15, align='right')
        # 5 Tipo de comprobante, fijo en RC
        ret += 'RC'
        # 6 Numero de comprobante Num, 15, AAAAMM rellenado con ceros a la izquierda
        anio = self.liquidacion.get('periodo_anio')
        mes = self.liquidacion.get('periodo_mes')
        periodo = f'{anio}{mes:02}'
        ret += f'{periodo:015}'
        # 7 Reservado para usos futuros, num, 4
        ret += '0000'
        # 8 Nombre del beneficiario, Alf, 30
        nombre_completo = empleado.get('nombre') + ' ' + empleado.get('apellido')
        ret += fixed_width_str(nombre_completo, 30)
        # 9 Direccion del beneficiario, Alf, 51
        ret += fixed_width_str(empleado.get('direccion'), 51)
        # 10 Reservado para usos futuros, Num, 5
        ret += '00000'
        # 11 Reservado para usos futuros, Alf, 4
        ret += ' ' * 4
        # 12 Reservado para usos futuros, Num, 83
        ret += '0' * 83
        # 13 Reservado para usos futuros, Alf, 11
        ret += ' ' * 11
        # 14 CUIL del beneficiario, Num, 11
        ret += empleado.get('cuil')
        # 15 Reservado para usos futuros, Alf, 162
        ret += ' ' * 162
        # 16 Marca de Agrup de beneficiarios, fijo="N"
        ret += 'N'
        # 17 Pais, fijo 0054
        ret += '0054'
        # TODO esto no esta listo
        # 18 CBU beneficiario. 26 caracteres, Diferente para Santender u otros
        ret += self.get_codigo_cbu(empleado)
        # 19 Reservado para usos futuros, Num, 8
        ret += '0' * 8
        # 20 Fecha de pago, Num, 8, AAAAMMDD
        anio = int(self.liquidacion.get('fecha_pago_anio'))
        mes = int(self.liquidacion.get('fecha_pago_mes'))
        dia = int(self.liquidacion.get('fecha_pago_dia'))
        full_dia = f'{anio}{mes:02}{dia:02}'
        ret += full_dia
        # 21 Import del pago, Num, 15, (13 enteros y 2 decimales sin separador)
        # Nuestro importe viene en string aun siedo un numero para que no se pierda precision
        parte_entera, parte_decimal = str_dec_num_to_no_dec_sep(empleado.get('importe_pago'))
        # asegurarse que la parte entera tiene 13 digitos y la parte decimal tiene 2 digitos con format string
        numero_sin_separa = f'{parte_entera:013}{parte_decimal:02}'
        ret += numero_sin_separa
        # 22 Codigo de forma de pago, Num, 2, 50=Acredit Santander, 52=SNP (otros bancos), 57=CCI (Otros bancos)
        ret += self.get_codigo_forma_pago()
        # 23 Reservado para usos futuros, Alf, 3
        ret += ' ' * 3
        # 24 Reservado para usos futuros, Num, 11
        ret += '0' * 11
        # 25 Reservado para usos futuros, Alf, 3
        ret += ' ' * 3
        # 26 Reservado para usos futuros, Num, 11
        ret += '0' * 11
        # 27 Reservado para usos futuros, Alf, 3
        ret += ' ' * 3
        # 28 Reservado para usos futuros, Num, 11
        ret += '0' * 11
        # 29 Reservado para usos futuros, Alf, 3
        ret += ' ' * 3
        # 30 Reservado para usos futuros, Num, 25
        ret += '0' * 25
        # 31 Reservado para usos futuros, Alf, 1
        ret += ' '
        # 32 Reservado para usos futuros, Num, 17
        ret += '0' * 17
        # 33 Reservado para usos futuros, Alf, 102
        ret += ' ' * 102
        return ret

    def get_codigo_cbu(self, empleado):
        """ Devuelve el CBU en formato especifico
            18 CBU beneficiario. 26 caracteres, Diferente para Santender u otros
            Santander:
             - Num(1) fijo 0
             - Num(3) Codigo de banco = 072 VER /acreditaciones/data/bancos.json
             - Num(4) Numero de sucursal
             - Num(1) Digito verificador Bloque 1
             - Num(3) Fijo 000
             - Num(1) Tipo de cuenta: 2=cta cte, 3=caja ahorro, 8=infinity
             - Num(1) Moneda: 0=pesos, 8=bimonetaria
             - Num(11) Numero de cuenta
             - Num(1) Digito verificador Bloque 2
            Otros bancos:
             - Num(1) fijo 0
             - Num(8) Bloque 1
             - Num(3) fijo 000
             - Num(14) Bloque 2
            EN RESUMEN. ES UN CBU COMUN QUE EMPIEZA CON UN 0 Y METE ADEMAS 3 CEROS FIJOS ANTES DEL SEGUNDO BLOQUE
            Al parecer tenian miedo de que se les acaben los numeros de cuenta.
            Ejemplo real Santander
            072 0469 6 8 8 00003516919 0

             Wikipedia: https://es.wikipedia.org/wiki/Clave_Bancaria_Uniforme
                La CBU está compuesta por 22 dígitos, separados en dos bloques. El primer bloque tiene un número de entidad
                de 3 dígitos, un número de sucursal de 4 dígitos y un dígito verificador. El segundo bloque tiene un número
                de 13 dígitos que identifica la cuenta dentro de la entidad y la sucursal, más un dígito verificador.
                [
                    XXX BANCO (ver tabla)
                    YYYY SUCURSAL
                    D DÍGITO VERIFICADOR
                    +
                    CTA[13] 13 digitos para la cuenta dentro del banco y sucursal
                    D DÍGITO VERIFICADOR
                ]
                Para el número de entidad de 3 dígitos que compone el primer bloque, debe verificarse el código de banco con
                la siguiente tabla de bancos
        """
        cbu = empleado.get('cbu')
        return '0' + cbu[0:9] + '000' + cbu[9:23]

    def generate_trailer(self):
        """ Generar el trailer """
        ret = 'T'  # 1 Tipo de registro = T trailer
        # 2 Reservado para usos futuros, Num, 15
        ret += '0' * 15
        # 3 Importe total del pago Num, 15, (13 enteros y 2 decimales sin separador)
        parte_entera, parte_decimal = str_dec_num_to_no_dec_sep(self.liquidacion.get('total_pago'))
        # asegurarse que la parte entera tiene 13 digitos y la parte decimal tiene 2 digitos con format string
        numero_sin_separa = f'{parte_entera:013}{parte_decimal:02}'
        ret += numero_sin_separa
        # 4 Cantidad de registros Num, 7
        total_empleados = len(self.empleados)
        ret += f'{total_empleados:07}'
        # 5 Reservado para usos futuros, Num, 612
        ret += '0' * 612
        return ret

    def get_nro_de_acuerdo(self):
        """ Devuelve un numero de acuerdo.
            Este es un codigo de dos digitos que se asigna por el banco.
            Se lo tenes que pedir a tu ejecutivo de cuenta.
        """

        return self.extras.get('nro_de_acuerdo', '00')

    def get_codigo_forma_pago(self):
        """ Devuelve el codigo de forma de pago """
        cfp = self.extras.get('codigo_forma_pago', '50')
        if cfp not in ['50', '52', '57']:
            raise ValueError('Código de forma de pago inválido')
        return cfp



import json
import os
import tempfile
import unittest

from py_arg_reports.reporters.f931.reporter import genera_txt_f931


class TestGeneracionF931(unittest.TestCase):
    """ Testing para Generación F931
    """
    @classmethod
    def setUpClass(cls):
        # Specify the path where the temporary folder should be created
        temp_folder_path = './tests/temp/'
        # Create a temporary folder for testing in the specified path
        cls.temp_folder = tempfile.mkdtemp(dir=temp_folder_path)
        if cls.temp_folder[-1] != '/':
            cls.temp_folder += '/'

        with open('./py_arg_reports/test_cases/f931-info.json', 'r') as f:
            cls.json_data = json.load(f)
        cls.empty_json = {}
        cls.key_missing_json = {
            "cuit": "30704552744",
            "periodo": "2024-04-01",
            "txt_empleados": [
                {
                    "cuil": "23223334441",
                    "nombre_completo": "GOMEZ, JUAN",
                    "conyuge": 0,
                    "hijos": 0,
                }
            ],
        }

    @classmethod
    def tearDownClass(cls):
        # Clean up: Delete the temporary folder and its contents
        if os.path.exists(cls.temp_folder):
            for root, dirs, files in os.walk(cls.temp_folder, topdown=False):
                for file in files:
                    os.remove(os.path.join(root, file))
                for dir in dirs:
                    os.rmdir(os.path.join(root, dir))
            os.rmdir(cls.temp_folder)

    def test_descarga_f931_1(self):
        """ Prueba de la generacion de txt de f931
        """
        resp_descarga = genera_txt_f931(
            json_data=self.json_data,
            output_path=self.temp_folder,
            filename='txt_f931_prueba_1',
        )

        self.assertEqual(resp_descarga, (True, None))
        full_path = self.temp_folder + 'txt_f931_prueba_1.txt'

        # Check if the file exists
        self.assertTrue(os.path.exists(full_path))

        # Check the number of lines in the txt file
        with open(self.temp_folder + 'txt_f931_prueba_1.txt', 'r', encoding='latin-1') as file:
            lines = file.readlines()
            num_lines = len(lines)
            largo_linea = len(lines[0]) if lines else 0

        # La línea debe tener 500 caracteres (incluyendo el salto de línea)
        # y el archivo 2 líneas por 2 empleados
        self.assertEqual(largo_linea, 500)
        self.assertEqual(num_lines, 2)

    def test_descarga_empty_json(self):
        """ Prueba la descarga del archivo para un json vacío
        """
        resp_descarga = genera_txt_f931(
            json_data=self.empty_json,
            output_path=self.temp_folder,
            filename='txt_f931_prueba_2',
        )

        self.assertEqual(resp_descarga, (False, 'No se puede generar el txt para el F931, no hay datos'))

        full_path = self.temp_folder + 'txt_f931_prueba_2.txt'

        # Check if the file not exists
        self.assertFalse(os.path.exists(full_path))

    def test_key_missing_json(self):
        """ Prueba la descarga del archivo para un json con datos faltantes
        """
        resp_descarga = genera_txt_f931(
            json_data=self.key_missing_json,
            output_path=self.temp_folder,
            filename='txt_f931_prueba_3',
        )

        self.assertEqual(resp_descarga, (False, 'El campo situacion no fue encontrado en los datos'))


if __name__ == '__main__':
    unittest.main()




from py_arg_reports.tools.tools import convert_to_float_if_possible


def process_reg3(cuil: str, concepto_liq: dict) -> tuple:
    """ Generacion txt de Libro Sueldos Digital
        Registro 3 Conceptos de sueldo liquidados al trabajador
        Args:
            cuil: str, CUIL del trabajador
            concepto_liq: dict, conceptos liquidados al trabajador
            Retorna:
            (Resultado, Error, Registro3)
            - True, None, str: si todo salió bien
            - False, str, None: si falla
    """
    resp = []
    key_esenciales_concepto = ['concepto', 'cantidad', 'importe']

    for concepto in concepto_liq:
        # Validaciones
        for key in key_esenciales_concepto:
            if key not in concepto:
                return False, f'Falta la clave {key} en el diccionario de conceptos', None

        cod_con = concepto['concepto'].ljust(10)
        this_cantidad = concepto['cantidad']
        this_importe = concepto['importe']
        if this_cantidad == '':
            this_cantidad = 0

        if this_importe == '':
            this_importe = 0

        try:
            this_cantidad = convert_to_float_if_possible(this_cantidad)
            this_importe = convert_to_float_if_possible(this_importe)
        except ValueError:
            return False, 'Error al convertir cantidad o importe a float', None

        temp_cant = str(round(this_cantidad * 100))[:5]
        cantidad = temp_cant.zfill(5)
        importe = round(abs(this_importe), 2) * 100
        importe = str(int(importe)).zfill(15)
        # Por default es Crédito
        if this_importe < 0:
            tipo = 'D'
        else:
            tipo = 'C'

        # Genero fila
        # Por ahora todo en unidades días
        # TODO: período ajuste retroactivo
        # https://github.com/Artimezoft/payroll_arg_reportes/issues/21
        item = f'03{cuil}{cod_con}{cantidad}D{importe}{tipo}{" " * 6}'
        resp.append(item)

    resp_final = '\r\n'.join(resp)

    return True, None, resp_final



from pathlib import Path
import json
import unittest
from PyPDF2 import PdfReader
from py_arg_reports.reporters.libro_sueldo import descargar_libro


class TestLibroSueldo(unittest.TestCase):
    """ Testing para Descargar libro sueldo """

    def setUp(self):
        self.samples_folder = 'py_arg_reports/reporters/libro_sueldo/samples'
        self.temp_folder = Path(self.samples_folder) / 'temp/'

    def test_descarga_recibo_1(self):
        """ Prueba la descarga del archivo """
        sample = 'samples-recibo-info.json'
        sample_path = Path(self.samples_folder) / sample
        json_data = json.load(open(sample_path))
        ok, error = descargar_libro(
            json_data=json_data,
            output_path=self.temp_folder,
            filename='pdf1',
        )

        self.assertIsNone(error)
        self.assertTrue(ok)
        expected_path = Path(self.temp_folder) / 'pdf1.pdf'

        # Check if the file exists
        self.assertTrue(expected_path.exists())

        # Check content
        f = open(str(expected_path), 'rb')
        # Check the number of pages in the PDF file
        pdf = PdfReader(f)
        pages = len(pdf.pages)
        self.assertEqual(pages, 1)

        # Probar que el contenido del archivo sea el esperado
        page = pdf.pages[0]
        page_1_text = page.extract_text()
        print("page_1_text", page_1_text)
        expected = [
            'Empresa de Prueba',
            'Calle de Prueba 123, 10, Formosa',
            'Actividad de Prueba',
            'Cultivo de flores',
            'Cultivo de plantas ornamentales',
            'No Remunerativos',
            # Agregue cosas y ya no se el total 'Neto a cobrar $ 329.181,77'
        ]
        for text in expected:
            self.assertIn(text, page_1_text)



import json

from py_arg_reports.reporters.libro_sueldo_digital.reporter import genera_txt_lsd


with open('./test_cases/lsd-info-1l.json', 'r') as f:
    lsd_data = json.load(f)

cuit_empresa = lsd_data['cuit']
periodo = lsd_data['periodo']

# Llamo a la función genera_txt_lsd con los datos del archivo lsd-info.json
resultado_descarga = genera_txt_lsd(
    json_data=lsd_data,
    output_path='./test_cases/downloads/',
    filename=f'{cuit_empresa}_{periodo}',
)

if resultado_descarga[0]:
    print('Descarga exitosa')
else:
    print('Error:', resultado_descarga[1])



import json
from py_arg_reports.reporters.libro_sueldo import descargar_libro


with open('./test_cases/liquidacion_completa.json', 'r') as f:
    liquidacion = json.load(f)

# Llamo a la función descargar_recibo
libro = descargar_libro(
    json_data=liquidacion,
    output_path='./downloads/',
    filename='libro_prueba',
)




def process_reg5() -> tuple:
    """ TODO: Pendiente para 2026 aproximadamente
        https://github.com/Artimezoft/payroll_arg_reportes/issues/20
    """

    resp = ""

    return True, None, resp



import datetime


def process_reg2(empleados_liquidados: dict, fecha_pago: datetime.date) -> tuple:
    """ Generacion txt de Libro Sueldos Digital
        Registro 2 Datos referenciales de la Liquidación de SyJ del trabajador
        Args:
            empleados_liquidados: dict, datos de los empleados liquidados
            fecha_pago: datetime.date, fecha de la liquidacion
            Retorna:
            (Resultado, Error, Registro2)
            - True, None, str: si todo salió bien
            - False, str, None: si falla
    """
    resp = []
    keys_esenciales_empleado = ['empleado_legajo', 'info_f931', 'conceptos_liquidados']
    keys_esenciales_f931 = ['fijos', 'variables', 'importes']
    keys_esenciales_f931_fijos = ['cuil', 'forma_pago', 'cbu', 'area']
    # Formas de pago aceptadas
    # '1'=Efectivo; '2'=Cheque; '3'=Acreditación en cuenta

    for info_empleado in empleados_liquidados:
        # Validaciones
        for key in keys_esenciales_empleado:
            if key not in info_empleado:
                return False, f'Falta la clave {key} en el diccionario de empleados', None

        for key in keys_esenciales_f931:
            if key not in info_empleado['info_f931']:
                return False, f'Falta la clave {key} en el diccionario de F931', None

        for key in keys_esenciales_f931_fijos:
            if key not in info_empleado['info_f931']['fijos']:
                return False, f'Falta la clave {key} en el diccionario de F931 Fijos', None

        legajo = info_empleado['empleado_legajo']
        cuil = info_empleado['info_f931']['fijos']['cuil']
        cbu = info_empleado['info_f931']['fijos']['cbu']
        area = info_empleado['info_f931']['fijos']['area']
        forma_pago = info_empleado['info_f931']['fijos']['forma_pago']
        fecha_rubrica = " " * 8

        if forma_pago not in ['1', '2', '3']:
            return False, f'Forma de pago no válida para el empleado {legajo}', None

        if forma_pago == '3' and (not cbu or len(cbu) != 22):
            return False, f'Forma de pago Acreditación en cuenta CBU inválido {legajo}', None

        if forma_pago != '3':
            cbu = " " * 22

        # Normalizacion
        legajo_norm = str(legajo).zfill(10)
        area_norm = " " * 50 if not area else area.ljust(50)
        fecha_pago_str = fecha_pago.strftime('%Y%m%d')

        item = f'02{cuil}{legajo_norm}{area_norm}{cbu}030{fecha_pago_str}{fecha_rubrica}{forma_pago}'

        resp.append(item)

    # Listo los registros, los uno con \r\n
    resp_final = '\r\n'.join(resp)

    return True, None, resp_final



import datetime
import os

from py_arg_reports.reporters.libro_sueldo_digital.generador_registros import (
    process_reg1,
    process_reg2,
    process_reg3,
    process_reg4,
    process_reg5,
)
from py_arg_reports.tools.tools import file_compress, delete_list_of_liles


def genera_txt_lsd_liquidacion(
    cuit: str,
    periodo: datetime.date,
    liquidacion_data: dict,
    nro_liq: int,
) -> tuple:
    """ Genera el archivo de texto para la generación del Libro Sueldo Digital
        Para una liquidación en particular
        Args:
            cuit: str, CUIT del empleador
            liquidacion_data: dict, datos de liquidación

        Retorna una tupla:
         - (Error bool, Error str, txt_data str)
         - False, error, None: si falla
         - True, None, txt_data: si todo salió bien
    """
    res_final = ''
    empleados_liquidados = liquidacion_data['empleados_liquidados']
    q_empleados = len(empleados_liquidados)
    tipo_liquidacion = liquidacion_data['tipo_liquidacion']
    fecha_pago = liquidacion_data['fecha_pago']
    fecha_pago_date = datetime.datetime.strptime(fecha_pago, '%Y-%m-%d').date()

    # Registro 1 -----------------------------------------------------------------------------------------
    registro1_txt = process_reg1(
        cuit=cuit,
        periodo=periodo,
        q_empleados=q_empleados,
        nro_liq=nro_liq,
        tipo_liq=tipo_liquidacion,
    )
    if not registro1_txt[0]:
        return False, registro1_txt[1], None

    res_final += registro1_txt[2]
    # Fin Registro 1 -------------------------------------------------------------------------------------

    # Registro 2 -----------------------------------------------------------------------------------------
    registro2_txt = process_reg2(
        empleados_liquidados=empleados_liquidados,
        fecha_pago=fecha_pago_date,
    )
    if not registro2_txt[0]:
        return False, registro2_txt[1], None
    # Agrego salto de linea
    res_final += '\r\n'
    res_final += registro2_txt[2]
    # Fin Registro 2 -------------------------------------------------------------------------------------

    # Registro 3 y 4 -------------------------------------------------------------------------------------
    registro_3_por_empleado = []
    registro_4_por_empleado = []
    for empleado in empleados_liquidados:
        # Agrupo toda la informacion de info_931
        info_931 = {}
        info_931.update(empleado["info_f931"]["fijos"])
        info_931.update(empleado["info_f931"]["variables"])
        info_931.update(empleado["info_f931"]["importes"])

        cuil = info_931['cuil']
        conceptos_liquidados = empleado['conceptos_liquidados']
        this_registro3 = process_reg3(
            cuil=cuil,
            concepto_liq=conceptos_liquidados,
        )
        this_registro4 = process_reg4(
            cuil=cuil,
            info_931=info_931,
        )
        if not this_registro3[0]:
            return False, this_registro3[1], None
        registro_3_por_empleado.append(this_registro3[2])

        if not this_registro4[0]:
            return False, this_registro4[1], None
        registro_4_por_empleado.append(this_registro4[2])

    registro3_txt = '\r\n'.join(registro_3_por_empleado)
    registro4_txt = '\r\n'.join(registro_4_por_empleado)
    # Agrego salto de linea
    res_final += '\r\n'
    res_final += registro3_txt
    # Agrego salto de linea
    res_final += '\r\n'
    res_final += registro4_txt
    # Fin Registro 3 y 4 ---------------------------------------------------------------------------------

    # Registro 5 -----------------------------------------------------------------------------------------
    registro5_txt = process_reg5()
    if not registro5_txt[0]:
        return False, registro5_txt[1], None
    if registro5_txt[2] != '':
        # Agrego salto de linea
        res_final += '\r\n'
    res_final += registro5_txt[2]
    # Fin Registro 5 -------------------------------------------------------------------------------------

    return True, None, res_final


def genera_archivo_final_lsd(txt_liquidaciones: list, output_path: str, filename: str) -> tuple:
    """ Genera el archivo de texto para la generación del Libro Sueldo Digital
        Args:
            txt_liquidaciones: list, lista de txt de liquidaciones
            output_path: str, ruta donde se guardará el archivo
            filename: str, nombre del archivo

        Retorna una tupla:
         - False, error, None: si falla
         - True, None, file_path: si todo salió bien
    """
    resp = None
    txt_liquidaciones_files = []

    # Voy escribiendo cada una de las liquidaciones
    for i, txt_liquidacion in enumerate(txt_liquidaciones):
        fpath = os.path.join(output_path, f'{filename}_{i+1}.txt')

        with open(fpath, 'w') as f:
            f.write(txt_liquidacion)

        txt_liquidaciones_files.append(fpath)

    if len(txt_liquidaciones) == 1:
        final_fpath = fpath
    else:
        zip_output_file_name = fpath = os.path.join(output_path, f'{filename}.zip')
        file_compress(txt_liquidaciones_files, zip_output_file_name)
        final_fpath = zip_output_file_name

        # Borro txts
        delete_list_of_liles(txt_liquidaciones_files)

    return True, resp, final_fpath


def genera_txt_lsd(
    json_data: dict,
    output_path: str,
    filename: str = None,
) -> tuple:
    """ Genera el archivo de texto para la generación del Libro Sueldo Digital
        Args:
            json_data: dict, datos de F931 Fijos
            output_path: str, ruta donde se guardará el archivo
            filename: str, nombre del archivo. En caso una liquidación genera txt, sino se genera un zip
            con todos los archivos de liquidaciones

        Retorna una tupla:
         - False, error: si falla
         - True, None: si todo salió bien
    """
    if not json_data:
        return False, 'No se puede generar el txt para Libro Sueldo Digital, no hay datos'

    resp = None
    indices_esenciales = ['periodo', 'cuit', 'cantidad_liquidaciones', 'liquidaciones']
    indices_esenciales_liquidaciones = ['tipo_liquidacion', 'fecha_pago', 'empleados_liquidados']

    # Validar que json_data tenga los datos esenciales
    for indice in indices_esenciales:
        if not json_data.get(indice):
            return False, f'No se puede generar el txt, dato esencial {indice} no encontrado'

    # Tomo las variables esenciales
    cuit = json_data['cuit']
    periodo = json_data['periodo']
    periodo_date = datetime.datetime.strptime(periodo, '%Y-%m-%d').date()
    liquidaciones = json_data['liquidaciones']

    # Valido Liquidaciones
    # 1) Que sea una lista de diccionarios
    # 2) Que tenga los datos esenciales
    if not isinstance(liquidaciones, list):
        return False, 'No se puede generar el txt, liquidaciones no es una lista de diccionarios'

    # Inicio nro_liquidacion
    nro_liquidacion = 0
    txt_liquidaciones = []
    for liquidacion in liquidaciones:
        if not isinstance(liquidacion, dict):
            return False, 'No se puede generar el txt, liquidaciones no es una lista de diccionarios'

        for indice in indices_esenciales_liquidaciones:
            if not liquidacion.get(indice):
                return False, f'No se puede generar el txt, dato esencial {indice} no encontrado en liquidaciones'

        nro_liquidacion += 1
        txt_liquidacion = genera_txt_lsd_liquidacion(
            cuit=cuit,
            periodo=periodo_date,
            liquidacion_data=liquidacion,
            nro_liq=nro_liquidacion,
        )
        if not txt_liquidacion[0]:
            return False, txt_liquidacion[1]

        txt_liquidaciones.append(txt_liquidacion[2])

    # Genero el archivo final
    resultado_final = genera_archivo_final_lsd(
        txt_liquidaciones=txt_liquidaciones,
        output_path=output_path,
        filename=filename,
    )
    if not resultado_final[0]:
        return False, resultado_final[1]

    return True, resp



import logging
from py_arg_reports.reporters.acreditaciones import AcreditacionFile
from py_arg_reports.reporters.descarga_excel import descarga_excel


log = logging.getLogger(__name__)


class AcreditacionGalicia(AcreditacionFile):
    """ Un archivo para el banco Santander """
    def __init__(self, data):
        super().__init__(
            data,
            empresa_required=False,
            liquidacion_required=False,
            empleados_require=['nombre', 'apellido', 'nro_cuenta', 'importe_pago'],
        )
        # Galicia es muy simple, posiblemente toma los datos desde el sistema del banco
        # Aqui solo pasamos un Excel con los datos de los empleados

    def load_data(self):
        """ Cargar solo los datos de empleados """
        self.load_empleados(require=self.empleados_require)

    def generate_file(self, path):
        """ Generar el archivo Excel que espera Galicia """
        data = {
            'headers': {
                'header1': {
                    'name': 'Cuenta',
                    'format': {'font_name': 'Arial', 'font_size': 12}
                },
                'header2': {
                    'name': 'Nombre',
                    'format': {'font_name': 'Arial', 'font_size': 12}
                },
                'header3': {
                    'name': 'Importe',
                    'format': {'num_format': '0.00', 'font_name': 'Arial', 'font_size': 12},
                    'is_number': True,
                },
                'header4': {
                    'name': 'Concepto',
                    'format': {'num_format': '0', 'font_name': 'Arial', 'font_size': 12},
                    'is_number': True,
                },
            },
            'data': []
        }

        for empleado in self.empleados:
            nombre = empleado.get('nombre', '')
            apellido = empleado.get('apellido', '')
            nro_cuenta = empleado.get('nro_cuenta')
            importe_pago = empleado.get('importe_pago')
            if isinstance(importe_pago, str):
                importe_pago = float(importe_pago)
            emp_dict = {
                'header1': nro_cuenta,
                'header2': f'{nombre} {apellido}',
                'header3': importe_pago,
                'header4': 1,
            }
            data['data'].append(emp_dict)

        try:
            excel_data = descarga_excel(data, sheet_name='Liquidaciones')
        except Exception as e:
            error = f'Error al crear el archivo Excel de acreditaciones Galicia: {e}'
            # trace
            import traceback
            full_error = error + '\n' + traceback.format_exc()
            log.error(full_error)
            return False, error

        f = open(path, 'wb')
        f.write(excel_data)
        f.close()
        return True, None



import datetime

from py_arg_reports.reporters.libro_sueldo_digital.tools import INDICE_TIPO_LIQUIDACIONES


def process_reg1(cuit: str, periodo: datetime.date, q_empleados: int, nro_liq: int, tipo_liq: str) -> tuple:
    """ Generacion txt de Libro Sueldos Digital
        Registro 1 Datos referenciales del envío de la liquidacion
        Args:
            cuit: str, CUIT del empleador
            periodo: datetime.date, fecha de la liquidacion
            q_empleados: int, cantidad de empleados liquidados
            nro_liq: int, numero de liquidacion
            tipo_liq: str, tipo de liquidacion ('M'=mes; 'Q'=quincena; 'S'=semanal)
        Retorna:
            (Resultado, Error, Registro1)
            - True, None, str: si todo salió bien
            - False, str, None: si falla
    """

    # Período en formato AAAAMM
    periodo_str = periodo.strftime('%Y%m')
    tipo_liq_aceptadas = INDICE_TIPO_LIQUIDACIONES.keys()

    # Validaciones
    cuit = cuit.replace('-', '')
    if len(cuit) != 11:
        return False, 'CUIT incorrecto', None

    if q_empleados < 1:
        return False, 'Cantidad de empleados incorrecta', None

    if nro_liq < 1:
        return False, 'Numero de liquidacion incorrecto', None

    if tipo_liq not in tipo_liq_aceptadas:
        return False, 'Tipo de liquidacion no aceptada', None

    tipo_liquidacion = INDICE_TIPO_LIQUIDACIONES[tipo_liq]

    # Pendiente tipo de liquidacion rectificativa
    resp = f'01{cuit}SJ'
    resp += periodo_str
    resp += tipo_liquidacion
    resp += str(nro_liq).zfill(5)

    ds_base = 30
    resp += str(ds_base).zfill(2) + str(q_empleados).zfill(6)

    return True, None, resp



"""
Process data from origin to what we need
"""
from py_arg_reports.tools.recibos_utils import (
    float_to_format_currency,
    formatted_date_str,
)


def translate_data(origin_data):

    resp = {}
    # Tomo los datos de la primera empresa (espero que sean todas iguales)
    if len(origin_data) == 0:
        raise ValueError("No hay datos para procesar (se espera al menos un empleado)")
    empresa = origin_data[0]['empresa']

    liquidacion = origin_data[0]['liquidacion']
    empleados = [emp for emp in origin_data]
    legajos = [emp['empleado']['legajo'] for emp in empleados]
    # Veo lo que realmente necesita el PDF
    resp = {
        'company_name': get_company_name(empresa),
        'domicilio': get_domicilio_empresa(empresa),
        'cuit': get_cuit(empresa),
        'actividad_principal': get_actividad_principal(empresa),  # TODO
        'actividades_secundarias': get_actividad_secundarias(empresa),
        'tipo_liquidacion': get_tipo_liquidacion(liquidacion),
        'periodo': get_periodo_liquidacon(liquidacion),
        'legajos': legajos,
        'empleados': []
    }
    for emp in empleados:
        empleado = emp['empleado']
        conceptos_liquidados = emp['conceptos_liquidados']
        liquidacion = emp['liquidacion']
        totales_liquidacion = emp['totales_liquidacion']
        resp['empleados'].append(
            {
                'nombre': get_nombre_empleado(empleado),
                'cuil': get_cuil_empleado(empleado),
                'legajo': empleado['legajo'],
                'estado_civil': get_estado_civil(empleado),
                'categoria': empleado.get('categoria', ''),
                'fecha_ingreso': get_nice_fecha(empleado.get('fecha_ingreso')),
                'fecha_ingreso_2': get_nice_fecha(empleado.get('fecha_ingreso_2')),
                'contrato': empleado.get('contrato', ''),
                'obra_social': empleado.get('obra_social', ''),
                'area': empleado.get('area', ''),
                'posicion': empleado.get('posicion', ''),
                'basico': get_empleado_basico(empleado),
                'lugar_trabajo': empleado.get('lugar_trabajo', ''),
                'conceptos_liquidados': get_conceptos_liquidados(conceptos_liquidados),
                'totales_liquidacion': totales_liquidacion,
                'relacion_bancaria': get_relacion_bancaria(empleado),
            },
        )
    return resp


def get_company_name(empresa):
    return empresa['name']


def get_domicilio_empresa(empresa):
    domicilio_obj = empresa['domicilio']
    domicilio = domicilio_obj.get('calle', '') + ' ' + domicilio_obj.get('numero', '')
    domicilio_elements_to_add = ['piso', 'oficina', 'barrio']
    for element in domicilio_elements_to_add:
        if domicilio_obj.get(element):
            domicilio += ', ' + domicilio_obj.get(element)

    localidad = domicilio_obj["localidad"]["name"]
    provincia = domicilio_obj["localidad"]["provincia"]["name"]
    domicilio += f', {localidad}, {provincia}'
    return domicilio


def get_cuit(empresa):
    return empresa['cuit']


def get_actividad_principal(empresa):
    # tomar el dato de actividad principal de la empresa
    return empresa['actividad_principal']


def get_actividad_secundarias(empresa):
    # tomar el dato de actividad secundaria de la empresa
    return empresa['actividades_secundarias']


def get_liquidacion(liquidacion):
    """ Obtener UNA liquidacion de un empleado """
    fecha_pago = formatted_date_str(liquidacion.get('fecha_pago'))
    ret = {
        'nro_liquidacion': liquidacion.get('nro_liquidacion'),
        'tipo_liquidacion': liquidacion.get('tipo_liquidacion'),
        'periodo': liquidacion.get('periodo', {}).get('periodo'),
        'fecha_pago': fecha_pago,
    }
    return ret


def get_tipo_liquidacion(liquidacion):
    """ Datos de liquidacion gewnerica (esperamos todas iguales) """
    return liquidacion['tipo_liquidacion']


def get_periodo_liquidacon(liquidacion):
    """ Datos de liquidacion gewnerica (esperamos todas iguales) """
    return liquidacion['periodo']['periodo']


def get_nombre_empleado(empleado):
    nombre = empleado['first_name']
    apellido = empleado['last_name']
    return f'{apellido}, {nombre}'


def get_cuil_empleado(empleado):
    cuil = empleado['cuil']
    return f'{cuil[:2]}-{cuil[2:10]}-{cuil[10:]}'


def get_conceptos_liquidados(conceptos):
    new_conceptos = []
    for concepto in conceptos:
        new_conceptos.append(
            {
                'code': concepto['concepto']['code'],
                'name': concepto['concepto']['name'],
                'tipo_concepto': concepto['concepto']['tipo_concepto'],
                'cantidad': concepto['cantidad'],
                'importe': concepto['importe'],
                'orden': concepto.get('orden', 99),
            }
        )

    return new_conceptos


def get_relacion_bancaria(empleado):
    return empleado.get('relacion_bancaria', {})


def get_empleado_basico(empleado):
    return float_to_format_currency(empleado.get('basico', 0))


def get_nice_fecha(fecha_str):
    if fecha_str:
        return formatted_date_str(fecha_str)
    return ''


def get_estado_civil(empleado):
    estado_civil = empleado.get('marital_status', 'Dato no disponible')
    return estado_civil




from py_arg_reports.reporters.libro_sueldo_digital.tools import FORMATO_TXT_LSD
from py_arg_reports.tools.tools import sync_format


KEYS_CALCULOS_ESPECIFICOS = [
    'base_calculo_diferencial_aporte_ss',
    'base_calculo_diferencial_contribucion_ss',
    'base_calculo_diferencial_lrt',
]


def process_keys_especificas(key_sp: str, info_931: dict) -> str:
    """ Procesa las claves específicas de los cálculos de la liquidación
        Para casos particulares que no vienen en Info F.931
    """
    # Por el momento no cambia ninguno, se usaba para base diferencial de AOS y COS
    # Pero finalmente se reporta en el campo correspondiente

    resp = "0"

    return resp


def process_reg4(cuil: str, info_931: dict) -> tuple:
    """ Generacion txt de Libro Sueldos Digital
        Registro 4 Datos referenciales de la Liquidación de SyJ del trabajador
        Args:
            cuil: str, CUIL del trabajador
            info_931: dict, datos de la liquidación
            Retorna:
            (Resultado, Error, Registro4)
            - True, None, str: si todo salió bien
            - False, str, None: si falla
    """
    resp = ''
    formato_txt_r4 = FORMATO_TXT_LSD['registro4']

    if len(cuil) != 11:
        return False, 'CUIL inválido', None

    for dato, formato in formato_txt_r4.items():
        long = formato['long']
        dato_type = formato['type']
        if dato == 'tipo_registro':
            dato_to_add = '04'
        elif dato == 'cuil':
            dato_to_add = cuil
        else:
            # Procesa las claves específicas
            if dato in KEYS_CALCULOS_ESPECIFICOS:
                dato_value = process_keys_especificas(dato, info_931)
            else:
                # Todos los demás items deben estar en info_931
                if dato not in info_931:
                    return False, f'Falta el campo {dato} en el diccionario de info_931', None
                dato_value = str(info_931[dato])
                # En el caso de decimal, se multiplica por 100 porque va sin coma
                multiplicador = 100 if dato_type == 'DE' else 1
                no_coma = True if dato_type == 'DE' else False
            dato_to_add = sync_format(dato_value, long, dato_type, multiplicador, no_coma)
        resp += dato_to_add

        if len(dato_to_add) != long:
            return False, f'Error en el largo del dato {dato}', None

    return True, None, resp



import json
import unittest
from pathlib import Path
from py_arg_reports.reporters.acreditaciones.santander import AcreditacionSantander


class TestAcreditacionSantander(unittest.TestCase):
    """ Testing para Acreditaciones Santander """

    def setUp(self):
        self.samples_folder = Path('py_arg_reports/reporters/acreditaciones/data')
        self.temp_folder = self.samples_folder / 'temp'
        # Crear la carpeta temporal
        self.temp_folder.mkdir(exist_ok=True)

    def test_acreditacion_santander_no_data(self):
        data = {}
        with self.assertRaises(ValueError):
            AcreditacionSantander(data=data)

    def test_acreditacion_santander_no_empresa(self):
        data = {
            'liquidacion': {
                'total_pago': '1000.00',
            }
        }
        with self.assertRaises(ValueError):
            AcreditacionSantander(data=data)

    def test_bad_codigo_de_pago(self):
        file_data = self.samples_folder / 'sample_santander_bad_codigo_de_pago.json'
        data = json.load(open(file_data))
        with self.assertRaises(ValueError) as e:
            AcreditacionSantander(data)
        expected = 'No hay un codigo_forma_pago'
        error = str(e.exception)
        self.assertIn(expected, error)

    def test_acreditacion_santander_ok(self):
        file_data = self.samples_folder / 'sample-santander.json'
        data = json.load(open(file_data))
        acreditacion = AcreditacionSantander(data)
        destination = self.temp_folder / 'santa.txt'
        process, _ = acreditacion.generate_file(destination)
        assert process
        # test results file
        f = open(destination)
        result_text = f.read()
        f.close()
        result_lines = result_text.split('\n')
        # Header + Detalles + Trailer
        self.assertEqual(len(result_lines), 1 + len(acreditacion.empleados) + 1)
        header = result_lines[0]
        self._test_header(header)
        trailer = result_lines[-1]
        self._test_trailer(trailer, acreditacion)
        detalles = result_lines[1:-2]
        for det in detalles:
            self._test_detalle(det)

    def _test_header(self, header):
        # El header empieza con H
        self.assertEqual(header[0], 'H')
        self.assertEqual(len(header), 650)
        cuit = header[1:12]
        # assert all numbers
        self.assertTrue(cuit.isdigit())
        self.assertEqual(header[12], '0')
        cod_productos = ['011', '013']
        cod_prod = header[13:16]
        self.assertIn(cod_prod, cod_productos)
        nro_acuerdo = header[16:18]
        nro_acuerdos_validos = ['88']
        self.assertIn(nro_acuerdo, nro_acuerdos_validos)
        cod_canal = header[18:21]
        self.assertEqual(cod_canal, '007')
        nro_envio = header[21:26]
        self.assertEqual(nro_envio, '00001')
        reservado = header[26:31]
        self.assertEqual(reservado, '00000')
        reservado = header[31:38]
        self.assertEqual(reservado, ' ' * 7)
        validacion_cuil = header[38]
        self.assertEqual(validacion_cuil, 'S')
        reservado = header[39:650]
        self.assertEqual(reservado, ' ' * 611)

    def _test_trailer(self, trailer, acreditacion):
        # El trailer empieza con T
        self.assertEqual(trailer[0], 'T')
        self.assertEqual(len(trailer), 650)
        # 15 ceros (reservado)
        self.assertEqual(trailer[1:16], '0' * 15)
        # 15 digitos, pago total
        total_pago = trailer[16:31]
        self.assertTrue(total_pago.isdigit())
        # 7 digitos para la cantidad de registros
        cant_registros = trailer[31:38]
        self.assertTrue(cant_registros.isdigit())
        can_registros_num = int(cant_registros)
        self.assertEqual(can_registros_num, len(acreditacion.empleados))
        # 612 ceros
        self.assertEqual(trailer[38:650], '0' * 612)

    def _test_detalle(self, det):
        """ Cada detalle es un empleado """
        # El detalle empieza con D
        self.assertEqual(det[0], 'D')
        self.assertEqual(len(det), 650)
        self.assertEqual(det[1], ' ')
        monedas = ['0', '2', '8']
        moneda = det[2]
        self.assertIn(moneda, monedas)
        # legajo = det[3:18]  # No hay validacion
        self.assertEqual(det[18:20], 'RC')
        anio = det[20:24]
        self.assertEqual(det[20:22], '20')
        self.assertTrue(anio.isdigit(), anio + det[18:30])
        mes = det[24:26]
        self.assertTrue(mes.isdigit())
        mes_num = int(mes)
        self.assertTrue(1 <= mes_num <= 12)
        # ceros hasta llegar a los 15 caracteres
        self.assertEqual(det[26:35], '0' * 9)
        self.assertEqual(det[35:39], '0000')
        nombre = det[39:69]
        self.assertTrue(nombre.strip())
        direccion = det[69:120]
        self.assertTrue(direccion.strip())
        self.assertEqual(det[120:125], '00000')
        self.assertEqual(det[125:129], ' ' * 4)
        self.assertEqual(det[129:212], '0' * 83)
        self.assertEqual(det[212:223], ' ' * 11)
        cuil = det[223:234]
        self.assertTrue(cuil.isdigit())
        self.assertEqual(det[234:396], ' ' * 162)
        self.assertEqual(det[396], 'N')
        self.assertEqual(det[397:401], '0054')
        codigo_cbu = det[401:427]
        self.assertTrue(codigo_cbu.isdigit())
        # el cbu empieza con cero
        self.assertEqual(codigo_cbu[0], '0')
        # Luego tiene 3 ceros fijos en la posicion 9, 10 y 11
        self.assertEqual(codigo_cbu[8:11], '000')
        self.assertEqual(det[427:435], '0' * 8)
        # fecha pago AAAAMMDD
        fecha_pago = det[435:443]
        self.assertTrue(fecha_pago.isdigit())
        self.assertEqual(fecha_pago[0:2], '20')
        mes = int(fecha_pago[4:6])
        self.assertTrue(1 <= mes <= 12)
        dia = int(fecha_pago[6:8])
        self.assertTrue(1 <= dia <= 31)
        # importe de pago, 15 digitos
        importe = det[443:458]
        self.assertTrue(importe.isdigit())
        cod_pagos_validos = ['50', '52', '57']
        cod_pago = det[458:460]
        self.assertIn(cod_pago, cod_pagos_validos)
        self.assertEqual(det[460:463], ' ' * 3)
        # 11 ceros (reservado)
        self.assertEqual(det[463:474], '0' * 11)
        # 3 espacios (reservado)
        self.assertEqual(det[474:477], ' ' * 3)
        # 11 ceros (reservado)
        self.assertEqual(det[477:488], '0' * 11)
        # 3 espacios (reservado)
        self.assertEqual(det[488:491], ' ' * 3)
        # 11 ceros (reservado)
        self.assertEqual(det[491:502], '0' * 11)
        # 3 espacios (reservado)
        self.assertEqual(det[502:505], ' ' * 3)
        # 25 ceros (reservado)
        self.assertEqual(det[505:530], '0' * 25)
        # Un espacio (reservado)
        self.assertEqual(det[530], ' ')
        # 17 ceros (reservado)
        self.assertEqual(det[531:548], '0' * 17)
        # 102 espacios
        self.assertEqual(det[548:650], ' ' * 102)
